<?php
/**
 * Cache
 * 
 * @package Struninn YouTube
 * 
 * @since 1.0.0
 * 
 * @author Odin Design Themes (https://odindesignthemes.com/)
 * 
 */

if (!class_exists('StruninnYouTube_Subcount_Cache')) {
  /**
   * Cache Manager.
   * 
   * @since 1.0.0
   */
  class StruninnYouTube_Subcount_Cache {
    private $file_path;

    private $cache_id;
    private $endpoint;
    private $endpoint_filters;

    private static $instance = false;

    private function __construct() {
      $cache_file_directory = STRUNINNYOUTUBE_SUBCOUNT_PATH . 'includes/functions/cache/';
      $cache_file_name = 'struninnyoutube-cache.json';
      $cache_file_path = $cache_file_directory . $cache_file_name;

      $this->file_path = $cache_file_path;

      $this->cache_id = false;
      $this->endpoint =  false;
      $this->endpoint_filters =  false;
    }

    /**
     * Returns class instance.
     * 
     * @since 1.0.0
     */
    public static function getInstance() {
      if (!self::$instance) {
        self::$instance = new StruninnYouTube_Subcount_Cache();
      }

      return self::$instance;
    }

    /**
     * Sets cache id.
     * 
     * @since 1.0.0
     * 
     * @param string $cache_id        Cache id.
     */
    public function setCacheID($cache_id) {
      $this->cache_id = $cache_id;
    }

    /**
     * Sets endpoint.
     * 
     * @since 1.0.0
     * 
     * @param string $endpoint        Cache endpoint.
     */
    public function setEndpoint($endpoint) {
      $this->endpoint = $endpoint;
    }

    /**
     * Sets endpoint filters.
     * 
     * @since 1.0.0
     * 
     * @param string $endpoint_filters        Cache endpoint filters.
     */
    public function setEndpointFilters($endpoint_filters) {
      $this->endpoint_filters = $endpoint_filters;
    }

    /**
     * Returns cache file content.
     * 
     * @since 1.0.0
     * 
     * @return array|bool $file_content       False if file doesn't exist, file content otherwise.
     */
    private function getFile() {
      $file_content = false;

      if (file_exists($this->file_path)) {
        $file_content = json_decode(file_get_contents($this->file_path), true);      
      }

      return $file_content;
    }

    /**
     * Updates cache file content.
     * 
     * @since 1.0.0
     * 
     * @param array $file_content       Content to update cache file with.
     * @return int|bool $result         Number of bytes that were written to the file, or false on failure.
     */
    private function updateFile($file_content) {
      $result = file_put_contents($this->file_path, json_encode($file_content));

      return $result;
    }

    /**
     * Returns cached endpoint.
     * 
     * @since 1.0.0
     * 
     * @return array|bool $cached_endpoint          Cached endpoint, or false if no data found.
     */
    private function getEndpoint() {
      $cached_endpoint = false;

      if ($this->cache_id) {
        $cached_data = $this->getFile();

        // there is a saved cached file with data
        if ($cached_data) {
          $cache_id_has_data = array_key_exists($this->cache_id, $cached_data);
          $cache_id_has_endpoint = $cache_id_has_data && array_key_exists($this->endpoint, $cached_data[$this->cache_id]);

          // the cache id has data associated and the endpoint exists
          if ($cache_id_has_data && $cache_id_has_endpoint) {
            $cached_endpoint_items = $cached_data[$this->cache_id][$this->endpoint];

            // cached data has filters associated
            if ($this->endpoint_filters) {
              // search cached data for complete filters match
              foreach ($cached_endpoint_items as $cached_endpoint_item) {
                if (count(array_keys($this->endpoint_filters)) !== count(array_keys($cached_endpoint_item['filters']))) {
                  continue;
                }

                $filter_match_count = 0;

                foreach ($this->endpoint_filters as $filter_name => $filter_value) {
                  $filter_name_exists = array_key_exists($filter_name, $cached_endpoint_item['filters']);
                  $filter_is_equal = $filter_name_exists && ($cached_endpoint_item['filters'][$filter_name] === $filter_value);

                  if ($filter_is_equal) {
                    $filter_match_count++;
                  }
                }

                // if all filters match, the data was found
                if ($filter_match_count === count(array_keys($this->endpoint_filters))) {
                  $cached_endpoint = $cached_endpoint_item;
                  break;
                }
              }
            // cached data doesn't have filters associated, return no filters data if it exists
            } else {
              foreach ($cached_endpoint_items as $cached_endpoint_item) {
                if (count(array_keys($cached_endpoint_item['filters'])) === 0) {
                  $cached_endpoint = $cached_endpoint_item;
                  break;
                }
              }
            }
          }
        }
      }

      return $cached_endpoint;
    }

    /**
     * Returns cached endpoint resource.
     * 
     * @since 1.0.0
     * 
     * @param string $resource                                Name of the resource to retrieve.
     * @return array|bool $cached_endpoint_resource           Cached cached endpoint resource, or false if no resource found.
     */
    private function getEndpointResource($resource) {
      $cached_endpoint_resource = false;

      $cached_endpoint = $this->getEndpoint();

      if ($cached_endpoint && array_key_exists($resource, $cached_endpoint)) {
        $cached_endpoint_resource = $cached_endpoint[$resource];
      }

      return $cached_endpoint_resource;
    }

    /**
     * Returns cached endpoint data.
     * 
     * @since 1.0.0
     * 
     * @return array|bool $cached_endpoint_data     Cached endpoint data, or false if no data found.
     */
    public function getEndpointData() {
      $cached_endpoint_data = $this->getEndpointResource('data');

      return $cached_endpoint_data;
    }

    /**
     * Returns cached endpoint data creation date.
     * 
     * @since 1.0.0
     * 
     * @return array|bool $cached_endpoint_data_creation_date     Cached endpoint data creation date, or false if no data found.
     */
    private function getEndpointDataCreationDate() {
      $cached_endpoint_data_creation_date = $this->getEndpointResource('creation_date');

      return $cached_endpoint_data_creation_date;
    }

    /**
     * Update endpoint data in cache file.
     * 
     * @since 1.0.0
     * 
     * @param array $data             Data to insert into cache file.
     * @return int|bool $result       Number of bytes that were written to the file, or false on failure.
     */
    public function updateEndpointData($data) {
      $result = false;

      if ($this->cache_id) {
        $cached_data = $this->getFile();

        if (!$cached_data) {
          $cached_data = [];
        }

        $cache_id_has_data = array_key_exists($this->cache_id, $cached_data);

        // cache id doesn't have any data, create data structure for cache id
        if (!$cache_id_has_data) {
          $cached_data[$this->cache_id] = [];
        }

        $cache_id_has_endpoint = array_key_exists($this->endpoint, $cached_data[$this->cache_id]);

        // cache id doesn't have endpoint data, create data structure for cache id endpoint
        if (!$cache_id_has_endpoint) {
          $cached_data[$this->cache_id][$this->endpoint] = [];
        }

        $endpoint_data = [
          'creation_date' => (new DateTime())->format('Y-m-d H:i:s'),
          'filters'       => $this->endpoint_filters ? $this->endpoint_filters : [],
          'data'          => $data
        ];

        // no filters, replace no filter endpoint data, otherwise create it
        if (!$this->endpoint_filters) {
          $filter_endpoint_data_found = false;

          for ($i = 0; $i < count($cached_data[$this->cache_id][$this->endpoint]); $i++) {
            if (count(array_keys($cached_data[$this->cache_id][$this->endpoint][$i]['filters'])) === 0) {
              $cached_data[$this->cache_id][$this->endpoint][$i] = $endpoint_data;
              $filter_endpoint_data_found = true;
              break;
            }
          }

          if (!$filter_endpoint_data_found) {
            $cached_data[$this->cache_id][$this->endpoint][] = $endpoint_data;
          }

        // if filter exists, replace data, otherwise create it
        } else {
          $filter_match_count = 0;

          for ($i = 0; $i < count($cached_data[$this->cache_id][$this->endpoint]); $i++) {
            $filter_match_count = 0;

            foreach ($this->endpoint_filters as $filter_name => $filter_value) {
              $filter_name_exists = array_key_exists($filter_name, $cached_data[$this->cache_id][$this->endpoint][$i]['filters']);
              $filter_is_equal = $filter_name_exists && ($cached_data[$this->cache_id][$this->endpoint][$i]['filters'][$filter_name] === $filter_value);

              if ($filter_is_equal) {
                $filter_match_count++;
              } else {
                break;
              }
            }

            // if all filters match, replace data
            if ($filter_match_count === count($this->endpoint_filters)) {
              $cached_data[$this->cache_id][$this->endpoint][$i] = $endpoint_data;
              break;
            }
          }

          // filter match was not found, create data
          if ($filter_match_count !== count($this->endpoint_filters)) {
            $cached_data[$this->cache_id][$this->endpoint][] = $endpoint_data;
          }
        }

        $result = $this->updateFile($cached_data);
      }

      return $result;
    }

    /**
     * Returns cached endpoint data age.
     * 
     * @since 1.0.0
     * 
     * @param string $time_unit                       Unit of time to retrieve age value. One of: 'days', 'hours', 'minutes'.
     * @return int|bool $cached_data_age              Cached data age value in time unit, or false if no data found.
     */
    private function getEndpointDataAge($time_unit) {
      $cached_data_age = false;

      $cached_data_creation_date = $this->getEndpointDataCreationDate();

      if ($cached_data_creation_date) {
        $cached_data_creation_date_time = (new DateTime())->createFromFormat('Y-m-d H:i:s', $cached_data_creation_date);

        $cached_data_creation_date_time_diff = $cached_data_creation_date_time->diff(new DateTime());

        switch ($time_unit) {
          case 'days':
            $cached_data_age = $cached_data_creation_date_time_diff->days;
            break;
          case 'hours':
            $cached_data_age = ($cached_data_creation_date_time_diff->days * 24) + $cached_data_creation_date_time_diff->h;
            break;
          case 'minutes':
          default:
            $cached_data_age = ($cached_data_creation_date_time_diff->days * 24 * 60) + ($cached_data_creation_date_time_diff->h * 60) + $cached_data_creation_date_time_diff->i;
        }
      }

      return $cached_data_age;
    }

    /**
     * Returns wether cached endpoint data is stale.
     * 
     * @since 1.0.0
     * 
     * @param int     $valid_age                          Cache data valid age. Default: 1.
     * @param string  $time_unit                          Unit of time of the age value. One of: 'days', 'hours', 'minutes', 'weekly'. Default: 'days'.
     * @return bool   $cached_data_is_stale               True if cached endpoint data is stale, false otherwise.
     */
    public function isEndpointDataStale($valid_age = 1, $time_unit = 'days') {
      $cached_data_is_stale = true;

      if ($time_unit === 'weekly') {
        $cached_data_creation_date = $this->getEndpointDataCreationDate();

        if ($cached_data_creation_date) {
          // cache data is stale if it was created before monday this week
          $cached_data_is_stale = (new DateTime('monday this week')) > (new DateTime($cached_data_creation_date));
        }
      } else {
        $cached_data_age = $this->getEndpointDataAge($time_unit);
    
        if ($cached_data_age !== false) {
          $cached_data_is_stale = $cached_data_age >= $valid_age;
        }
      }

      return $cached_data_is_stale;
    }
  }
}

?>