<?php
/**
 * Functions - API
 * 
 * @package Struninn YouTube
 * 
 * @since 1.0.0
 * 
 * @author Odin Design Themes (https://odindesignthemes.com/)
 * 
 */

if (!function_exists('struninnyoutube_endpoint_parts_string_get')) {
  /**
   * Returns YouTube endpoint parts string.
   * 
   * @since 1.0.0
   * 
   * @param array   $parts            YouTube endpoint resource parts.
   * @return string $parts_string     YouTube endpoint resource parts string.
   */
  function struninnyoutube_endpoint_parts_string_get($parts = []) {
    $parts_string = '';

    if (count($parts) > 0) {
      $parts_string = implode(',', array_keys($parts));
  
      $parts_string .= '&fields=items(';
  
      $part_scopes = [];
  
      foreach ($parts as $part => $part_scope) {
        $part_scope_flat = $part;
  
        if (count($part_scope) > 0) {
          $part_scope_flat .= '(' . implode(',', $part_scope) . ')';
        }
  
        $part_scopes[] = $part_scope_flat;
      }
  
      $parts_string .= implode(',', $part_scopes);
  
      $parts_string .= ')';
    }

    return $parts_string;
  }
}

if (!function_exists('struninnyoutube_endpoint_data_get')) {
  /**
   * Returns YouTube endpoint resource data.
   * 
   * @since 1.0.0
   * 
   * @param string  $resource     YouTube endpoint resource.
   * @param array   $params       YouTube endpoint resource params.
   * @param array   $parts        YouTube endpoint resource parts.
   * @return array  $result       YouTube endpoint resource data.        
   */
  function struninnyoutube_endpoint_data_get($resource = '', $params = [], $parts = []) {
    $endpoint = 'https://www.googleapis.com/youtube/v3/';
    $endpoint_url = $endpoint . $resource;

    $access_keys = struninnyoutube_access_keys_get();

    if ($access_keys['api_key'] === '') {
      return struninnyoutube_error_invalid_api_key_get();
    }

    $channel_id = struninnyoutube_channel_id_get();

    if (!$channel_id) {
      return struninnyoutube_error_invalid_channel_id_get();
    }

    if (array_key_exists('id', $params) && ($params['id'] === '')) {
      return struninnyoutube_error_invalid_channel_id_get();
    }

    if (array_key_exists('channelId', $params) && ($params['channelId'] === '')) {
      return struninnyoutube_error_invalid_channel_id_get();
    }

    $endpoint_url .= '?key=' . $access_keys['api_key'];

    if (count($params) > 0) {
      $parameters = [];

      foreach ($params as $param_name => $param_value) {
        $parameters[] = $param_name . '=' . $param_value;
      }
    
      $endpoint_url .= '&' . implode('&', $parameters);
    }

    if (count($parts) > 0) {
      $endpoint_url .= '&part=' . struninnyoutube_endpoint_parts_string_get($parts);
    }

    // check if data exists in cache and is not stale before requesting endpoint
    $cache_endpoint = [
      'channels'      => 'subscriber_count',
      'search'        => 'video_list',
      'playlistItems' => 'video_list',
      'videos'        => 'video_list',
      'playlists'     => 'playlist_list'
    ];

    $cache_age_setting_name = array_key_exists($resource, $cache_endpoint) ? 'struninnyoutube_setting_' . $cache_endpoint[$resource] . '_cache_age' : false;
    $cache_unit_setting_name = array_key_exists($resource, $cache_endpoint) ? 'struninnyoutube_setting_' . $cache_endpoint[$resource] . '_cache_unit' : false;

    $cache_stale_age = $cache_age_setting_name ? absint(struninnyoutube_customizer_setting_value_get($cache_age_setting_name)) : 1;
    $cache_stale_unit = $cache_unit_setting_name ? struninnyoutube_customizer_setting_value_get($cache_unit_setting_name) : 'days';

    $cache = StruninnYouTube_Subcount_Cache::getInstance();
    $cache->setCacheID($channel_id);
    $cache->setEndpoint($resource);
    $cache->setEndpointFilters($params);

    $cache_data_is_stale = $cache->isEndpointDataStale($cache_stale_age, $cache_stale_unit);

    if (!$cache_data_is_stale) {
      return $cache->getEndpointData();
    }

    $response = wp_remote_get($endpoint_url);

    if (is_wp_error($response)) {
      $result = struninnyoutube_error_get($response);
    } else {
      if (is_array($response)) {
        $result = json_decode($response['body'], true);

        if (array_key_exists('items', $result)) {
          $result = $result['items'];

          // update cache
          $cache->updateEndpointData($result);
        } else if (array_key_exists('error')) {
          $result = $result['error'];
        }
      } else {
        $result = struninnyoutube_error_invalid_data_get();
      }
    }

    return $result;
  }
}

?>